#!/usr/bin/X11/tixwish
#  File: print.tcl
# 
#      This file is part of minkowsky
# 
#      Copyright (C) 2001-2002 by Rdiger Goetz
#      Author: Rdiger Goetz <minkowsky@r-goetz.de>
# 
#      Time-stamp: <05-Jun-2002 22:48:30 goetz>
# 
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
# 
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
# 
#      You should have received a copy of the GNU General Public License
#      along with this program; if not, write to the Free Software
#      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#  

proc initPrint { } {
    global printPageFormats printBaseFormats printPageFormat

    set printBaseFormats(A3P)  "785 1120 0"
    set printBaseFormats(A3L) "1120  785 1"
    set printBaseFormats(A4P)  "538  765 0"
    set printBaseFormats(A4L)  "765  538 1"
    set printBaseFormats(ltP)  "556  723 0"
    set printBaseFormats(ltL)  "723  556 1"
    set printBaseFormats(lgP)  "556  935 0"
    set printBaseFormats(lgL)  "935  556 1"

    set printPageFormats(A3P) "{A3 portrait}      A3P 27.7c 39.5c a3"
    set printPageFormats(A3L) "{A3 landscape}     A3L 39.5c 27.7c a3"
    set printPageFormats(A4P) "{A4 portrait}      A4P 19.0c 27.0c a4"
    set printPageFormats(A4L) "{A4 landscape}     A4L 27.0c 19.0c a4"
    set printPageFormats(A5P) "{A5 portrait}      A4P 12.8c 18.5c a5"
    set printPageFormats(A5L) "{A5 landscape}     A4L 18.5c 12.8c a5"
    set printPageFormats(ltP) "{Letter portrait}  ltP 19.6c 25.5c letter"
    set printPageFormats(ltL) "{Letter landscape} ltL 25.5c 19.6c letter"
    set printPageFormats(lgP) "{Legal portrait}   lgP 19.6c 33.0c legal"
    set printPageFormats(lgL) "{Legal landscape}  lgL 33.0c 19.6c legal"
#     set printPageFormats(A4p) "{A4 portrait} 538 765 19c 29c 0"
#     set printPageFormats(A4l) "{A4 landscape} 765 538 29c 19c 1"
    set printPageFormat A4P
    getPrinterList
    initPrintCal
    initPrintProj
    initAdrPrint
}

proc getPrinterList { } {
    global printerList stdPrinter env printerFullNames
    set fid [open "/etc/printcap" ]
    
    set capList ""
    set c [gets $fid line]
    set store 1
    while {! [eof $fid] } {
	if { $c > 0} {
	    set line [string trim $line]
	    set sc [string range  $line 0 0]
	    if { $sc != "\#" && $sc !=":" } {
		set p [ string first ":" $line ]
		if { $p>0} {
		    incr p -1
		    lappend capList [string range $line 0 $p]
		} else {
		    lappend capList $line
		}
	    }
	}
	set c [gets $fid line]
    }
#    puts "CapList: $capList"
    set printerList ""
    set i 0
    set stdPrinter 0
    set defPrinter ""
    set myPr ""
    catch { set myPr $env(PRINTER)}
    if { [ string length $myPr ] > 0 } { set defPrinter $myPr}

    set stdPrinter -1
    set stdPr -1
    foreach line $capList {
	set prl [split $line | ]
	set pr [ lindex $prl  0]

	if { [lsearch $printerList $pr] >=0 } {
	    warnAboutPrintCap $pr
	} else {
	    if { [ lsearch $prl $defPrinter] >=0 } {
		set stdPrinter $i
	    }
	    if { [ lsearch $prl "lp"] >=0 } {
		set stdPr $i
	    }
	    lappend printerList $pr
	    set printerFullNames($pr) [lindex $prl end]
	    incr i
	}
    }
    if { $stdPrinter < 0} {
	if { $stdPr>=0 } {
	    set stdPrinter $stdPr
	}
    }
    if { $stdPrinter < 0} {
	set stdPrinter 0
    }
}

proc warnAboutPrintCap { pr } {
    global bfnt fnt  _allgText _buttonText progname


    update 
    after 100
    if { [winfo exists .prw ] == 1 } { return }
    toplevel .prw
    
    frame .prw.t -relief raised -bd 1
    frame .prw.b 
    pack .prw.t .prw.b -side top -fill x -expand 1

    label .prw.t.l -text "$_allgText(Warn):" -fg red2 -font $bfnt
    message .prw.t.m -text "$_allgText(PrintCapWarn1) $pr $_allgText(PrintCapWarn2) $progname $_allgText(PrintCapWarn3)" -font $fnt -width 240
    pack .prw.t.l  .prw.t.m  -side top -fill x
    button .prw.b.ok -text "  $_buttonText(OK)  " -command {destroy .prw}
    pack .prw.b.ok -side right
    wm title .prw $_allgText(WARN)
    centerWin .prw 250 140
    update
   # puts [wm geometry .prw]
    tkwait window .prw
}


proc printDialog { } {
    global printerList printerFullNames stdPrinter selPrinter printMode
    global bfnt fnt fnt10 fnt12 toolSelection  bfnt12
    global _allgText _buttonText _printText 
    global printPageFormats printPageFormat

    if { [winfo exists .pr ] == 1 } { return }

    set printMode c
    
    if { [info exists selPrinter ] == 0 } {
	set selPrinter [lindex $printerList $stdPrinter ]
    }

    if { $selPrinter == "" } {
	set selPrinter [lindex $printerList $stdPrinter ]
    }

    toplevel .pr
    busyCursor .pr

    frame .pr.t -relief raised -bd 1
    frame .pr.c 
    frame .pr.b 
    pack .pr.t .pr.c  -side top -fill x 
    pack .pr.b -side top -fill x  -pady 2
    

    frame .pr.t.0
    frame .pr.t.1
    tixLabelFrame .pr.t.2 -label $_allgText(PrintType) -labelside acrosstop -padx 2 -pady 2
    tixLabelFrame .pr.t.3 -label $_allgText(Options) -labelside acrosstop -padx 0 -pady 0
    .pr.t.2 subwidget label configure -font $fnt10
    .pr.t.3 subwidget label configure -font $fnt10
    pack  .pr.t.0  .pr.t.1 -side top  -fill x -pady 4  -padx 4
    
    tixOptionMenu .pr.t.0.pr -label "$_allgText(Printer)" -options {menubutton.width 40 label.width 13}
    .pr.t.0.pr subwidget label configure -font $bfnt
    .pr.t.0.pr subwidget menubutton configure -font $bfnt12
    .pr.t.0.pr subwidget menu configure -font $bfnt12
    foreach printer $printerList {
 	.pr.t.0.pr add command $printer -label $printerFullNames($printer)
    }
    .pr.t.0.pr add separator fileSep
    .pr.t.0.pr add command nolp__file__ -label $_allgText(LabelPrint2File) 
    .pr.t.0.pr add command nolp__gv____ -label $_allgText(LabelPrint2FileShow) 
    .pr.t.0.pr configure -variable selPrinter
    pack .pr.t.0.pr -side left
    
    tixOptionMenu .pr.t.1.p -options {menubutton.width 40 label.width 13} \
 	-label "$_allgText(PrintPageFormat)" 
    .pr.t.1.p subwidget label configure -font $bfnt
    .pr.t.1.p subwidget menu configure -font $bfnt12
    .pr.t.1.p subwidget menubutton configure  -font $bfnt12
    foreach ent [lsort [array names printPageFormats ]] {
	.pr.t.1.p add command $ent  -label [lindex $printPageFormats($ent) 0]
    }
    .pr.t.1.p configure -variable printPageFormat
    pack .pr.t.1.p -side left -fill x


    

    switch $toolSelection {
	K {
	    printCalDialog .pr.t.2
	    printCalOptionsDialog .pr.t.3 
	    canvas  .pr.c.c -width 580 -height 866 -background white
	    centerWin .pr 436 424
	}
	A { 
	    printAdrDialog .pr.t.2
	    printAdrOptionsDialog .pr.t.3 
	    canvas  .pr.c.c -width 538 -height 765 -background white
	    centerWin .pr 450 722
	}
	T { 
	    printProjDialog .pr.t.2
	    printProjOptionsDialog .pr.t.3 
	    canvas  .pr.c.c -width 580 -height 866 -background white
	    centerWin .pr 440 220
	}
	- { 
	    destroy .pr
	    return
	}
    }
    

    button .pr.b.abb -text $_buttonText(Cancel) -command {closeWin .pr} -font $fnt -width 7
    pack .pr.b.abb -side left

    button .pr.b.pr -text $_buttonText(Print) -command printCurCanvas -font $bfnt -width 7
    pack .pr.b.pr -side right
    
    bind .pr <Return> printCurCanvas
    bind .pr <KP_Enter> printCurCanvas

    wm title .pr $_allgText(wmTitle_pr)
    update 
#    puts [wm geometry .pr]
    grab .pr
    stdCursor .pr
}





proc printCurCanvas { } {
    global  selPrinter printFile _buttonText printSelFildDlg
    
    set printFile ""
    if { $selPrinter == "nolp__file__" || $selPrinter ==  "nolp__gv____" } {

 	set dialog [tix filedialog tixFileSelectDialog]
 	$dialog config -command setPrinterFile
	$dialog subwidget fsbox configure -pattern *.ps
	$dialog subwidget fsbox subwidget dirlist subwidget vsb configure -width 12
	$dialog subwidget fsbox subwidget dirlist subwidget hsb configure -width 12
	$dialog subwidget fsbox subwidget filelist subwidget vsb configure -width 12
	$dialog subwidget fsbox subwidget filelist subwidget hsb configure -width 12

	set printSelFildDlg $dialog
	set bb [$dialog subwidget btns]
	set ok   [$bb subwidget ok]
	set filt [$bb subwidget apply]
	set hlp [$bb subwidget help]
	set abb [$bb subwidget cancel]
	$abb configure -text $_buttonText(Cancel)
	$abb configure -command {destroy $printSelFildDlg}
	pack forget $hlp $ok $filt $abb
	pack $abb -side left -padx 5 -pady 10
	pack $ok -side right  -padx 5 -pady 10
	pack $filt -side right  -padx 10 -pady 10
 	$dialog popup
	grab $dialog
    } else {
	_printCurCanvas	
    }
}
proc setPrinterFile {file} {
    global printFile printSelFildDlg _allgText

    set printFile $file

    if { $printFile == "" } {
	closeWin .pr
	_ErrorDialog $_allgText(PrintToFileErrorTitle) $_allgText(PrintToFileError)
	return
    }
    if {  [ file isdirectory $printFile] } {
	closeWin .pr
	_ErrorDialog $_allgText(PrintToFileErrorTitle) $_allgText(PrintToFileError)
	return
    }
    set dname [file dirname $printFile ]
    if { ! [ file exists $dname] } {
	closeWin .pr
	_ErrorDialog $_allgText(PrintToFileErrorTitle) $_allgText(PrintToFileError)
	return
    }
    if { ! [ file isdirectory $dname] } {
	closeWin .pr
	_ErrorDialog $_allgText(PrintToFileErrorTitle) $_allgText(PrintToFileError)
	return
    }
    
    if { [ file exists $printFile] } { 
	if { ! [ file writable $printFile] } {
	    closeWin .pr
	    _ErrorDialog $_allgText(PrintToFileErrorTitle) $_allgText(PrintToFileError)
	    return
	}
    } else { 
	if { ! [ file writable $dname] } {
	    closeWin .pr
	    _ErrorDialog $_allgText(PrintToFileErrorTitle) $_allgText(PrintToFileError)
	    return
	}
    }

    _printCurCanvas
}

proc freePrintCurCanvas { } {
    global printSelFildDlg 
    puts "freePrintCurCanvas: $printSelFildDlg "
    grab release $printSelFildDlg
    destroy $printSelFildDlg
    grab .pr
}
proc _printCurCanvas { } {
    global toolSelection   printMode printModeCal
    global  selPrinter cputype printMode  env printFile 
    global  printPageFormat printPageFormats _allgText


    if { $selPrinter == "nolp__file__" || $selPrinter ==  "nolp__gv____" } {
	set file $printFile
    } else {
 	set file [format "/tmp/minko-%d-%d.ps" [pid] [currentTime] ]
    }

    switch $toolSelection {
	K {
	    set printModeCal $printMode
	    printCal  $file
	}
	A {printAdrList $file }  
	T {printProjList $file}
#	T {printProjs}
  }
    
    update
#    after 50
    closeWin .pr
}


proc send2Printer { file } {
    global selPrinter printPageFormat printPageFormats

    set media [lindex $printPageFormats($printPageFormat) 4]
    if { $selPrinter ==  "nolp__gv____" } {
	set ret [ exec gv -media $media $file &]
    } elseif { $selPrinter != "nolp__file__"  } {
	set ret [exec lpr -P$selPrinter $file]	
	file delete $file
    }
}





proc stdCanvas2PS { cwin file  cwidth cheight colorType } {
    global printPageFormats printPageFormat printBaseFormats  
    global printColorMap


    set base    [ lindex $printPageFormats($printPageFormat) 1]
    set pwidth  [ lindex $printPageFormats($printPageFormat) 2]
    set pheight [ lindex $printPageFormats($printPageFormat) 3]
    
    set l [string length $pwidth]
    set pw [string range $pwidth 0 [expr $l -2]]
    set l [string length $pwidth]
    set ph [string range $pheight 0 [expr $l -2]]
    set wr [expr $cwidth / $pw ]
    set hr [expr $cheight/ $ph]


    set cmd "$cwin  postscript  -file $file -y 0 -pagex 1c -x 0 -width  $cwidth -height $cheight"
    if { $wr >= $hr  } { 
	set off [expr ($ph -($cheight/ $wr))/2 ]
	set off "[expr floor($off*10+.5)/10+1]c"
	set cmd "$cmd -pagex $off -pagey 1c -pageanchor nw -pagewidth $pwidth -rotate 1"
    } else {
	set off [expr ($pw -($cwidth / $hr))/2 ]
	set off "[expr floor($off*10+.5)/10+1]c"
	set cmd "$cmd -pagey $off -pagex 1c  -pageanchor nw -pageheight $pheight -rotate 1"
    }
    if {$colorType == "color" } {
	set cmd "$cmd  -colormode color"
    } else { 
	set cmd "$cmd  -colormode gray -colormap printColorMap"
    }
    
    if { $file != "-" } {
	set cmd "$cmd -file $file"
	eval $cmd
	return 1
    } 
    return [eval $cmd ]
}

proc canvas2PS { cwin file x y scale  } {
    global printPageFormats printPageFormat printBaseFormats  

    set base    [ lindex $printPageFormats($printPageFormat) 1]
    set cwidth  [expr [ lindex $printBaseFormats($base)   0] *$scale ]
    set cheight [expr [ lindex $printBaseFormats($base)   1] *$scale ]
    set rotate  [ lindex $printBaseFormats($base)   2]
    set pwidth  [ lindex $printPageFormats($printPageFormat) 2]
    set pheight [ lindex $printPageFormats($printPageFormat) 3]

    set cmd "$cwin  postscript  -colormode gray -pagey 1c -y $y -pagex 1c -x $x -width  $cwidth -height $cheight"
    if { $rotate =="0" } {
	set cmd "$cmd -pagewidth  $pwidth  -pageanchor sw -rotate $rotate"
    } else {
	set cmd "$cmd -pageheight $pheight -pageanchor nw -rotate $rotate"
    }
    if { $file != "-" } {
	set cmd "$cmd -file $file"
	eval $cmd
	return 1
    } 
    return [eval $cmd ]
}
