#!/usr/bin/X11/tixwish
#  File: printCal.tcl
# 
#      This file is part of minkowsky
# 
#      Copyright (C) 2001-2002 by Rdiger Goetz
#      Author: Rdiger Goetz <minkowsky@r-goetz.de>
# 
#      Time-stamp: <05-Jun-2002 14:05:45 goetz>
# 
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
# 
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
# 
#      You should have received a copy of the GNU General Public License
#      along with this program; if not, write to the Free Software
#      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#  

proc initPrintCal { } {
    global pw_skipWeekend pw_useColor
    set pw_skipWeekend 0
    set pw_useColor graymap
}

proc printCalDialog { win } {
    global printMode calTyp
    global bfnt fnt fnt10 fnt12 bfnt12
    global prStartD prStartM prStartY prStopD prStopM prStopY
    global curDay curMon curYear monlen
    global pwDay pwMon pwYear 
    global printModeCal  printPageFormat printPageFormats
    global pw_skipWeekend pw_useColor
    global printCalFormatWin
    global _allgText _buttonText
    
    set pwDay $curDay
    set pwMon $curMon
    set pwYear $curYear

    pack  $win  -side top  -fill x -padx 2 -pady 1 -expand 1
    set fwin [$win subwidget frame]
    set printCalFormatWin $fwin

    set printMode $printModeCal


    frame $fwin.1
    frame $fwin.2
    frame $fwin.3
    frame $fwin.4
    frame $fwin.5
    pack $fwin.1 $fwin.2 $fwin.3 $fwin.4 $fwin.5 -side top -fill x -padx 3 -pady 3


    
    radiobutton $fwin.1.r -variable printMode -value w -text $_allgText(PrintCalWeekView) -command togglePRdates -font $bfnt -width 20 -anchor  nw
    radiobutton $fwin.2.r -variable printMode -value m -text $_allgText(PrintCalMonthView) -command togglePRdates -font $bfnt -width 20 -anchor  nw
    radiobutton $fwin.3.r -variable printMode -value l -text $_allgText(PrintCalListView) -command togglePRdates -font $bfnt -width 20 -anchor  nw
    radiobutton $fwin.5.r -variable printMode -value c -text $_allgText(PrintCalCurView) -command togglePRdates -font $bfnt -width 20 -anchor  nw
    pack $fwin.1.r $fwin.2.r $fwin.3.r $fwin.5.r -side left

    tixControl $fwin.1.d  -label $_allgText(PrintCalVom) -min 0 -max 31 -integer 1  \
	-options {  entry.width 2 label.width 6 } -variable pwDay -selectmode immediate -autorepeat true
    tixControl $fwin.1.m -options { entry.width 2} -label "." -min 0 -max 12 -integer 1 -variable pwMon    -selectmode immediate -autorepeat true
    tixControl $fwin.1.y -options { entry.width 4} -label "." -min 1999 -max 2100 -integer 1  -variable pwYear   -selectmode immediate -autorepeat true
    pack $fwin.1.y $fwin.1.m $fwin.1.d -side right 
    $fwin.1.d subwidget label configure -font $bfnt
    $fwin.1.d subwidget entry configure -font $fnt
    $fwin.1.m subwidget label configure -font $bfnt
    $fwin.1.m subwidget entry configure -font $fnt
    $fwin.1.y subwidget label configure -font $bfnt
    $fwin.1.y subwidget entry configure -font $fnt
    $fwin.1.d  configure -state normal
    $fwin.1.m  configure -state normal
    $fwin.1.y  configure -state normal
    
    tixControl $fwin.2.m -options { entry.width 2 } -min 0 -max 12 -integer 1 -variable pwMon    -selectmode immediate -autorepeat true
    tixControl $fwin.2.y -options { entry.width 4} -label "." -min 1999 -max 2100 -integer 1  -variable pwYear   -selectmode immediate -autorepeat true
    pack $fwin.2.y $fwin.2.m  -side right 
    $fwin.2.m subwidget label configure -font $bfnt
    $fwin.2.m subwidget entry configure -font $fnt
    $fwin.2.y subwidget label configure -font $bfnt
    $fwin.2.y subwidget entry configure -font $fnt
    $fwin.2.m  configure -state disabled
    $fwin.2.y  configure -state disabled

    switch $calTyp {
	d { set prStartD $curDay
	    set prStartM $curMon
	    set prStartY $curYear
	    set prStopD $curDay
	    set prStopM $curMon
	    set prStopY $curYear
	}	
	w {
	    set shift  [ weekDayByDate $curDay $curMon $curYear]
	    set ret [calcDate $curDay $curMon $curYear [expr -1* $shift]]
	    set prStartD  [lindex $ret 0]
	    set prStartM  [lindex $ret 1]
	    set prStartY  [lindex $ret 2]
	    set ret [calcDate  $prStartD $prStartM $prStartY  7]
	    set prStopD  [lindex $ret 0]
	    set prStopM  [lindex $ret 1]
	    set prStopY  [lindex $ret 2]
	}
	m {
	    set prStartD  1
	    set prStartM $curMon
	    set prStartY $curYear
	    set shift  [ weekDayByDate 1 $curMon $curYear]
	    set ret [calcDate  $prStartD $prStartM $prStartY  [expr -1* $shift]]
	    set prStartD  [lindex $ret 0]
	    set prStartM  [lindex $ret 1]
	    set prStartY  [lindex $ret 2]

	    set prStopD [ lindex $monlen $curMon]
	    if {$curMon == 2} {
		if { [expr $curYear %4 ] == 0} {
		    incr prStopD
		}
	    }
	    set prStopM $curMon
	    set prStopY $curYear
	    set shift  [ weekDayByDate $prStopD $curMon $curYear]
	    set ret [calcDate  $prStopD $prStopM $prStopY  [expr 6-$shift]]
	    set prStopD  [lindex $ret 0]
	    set prStopM  [lindex $ret 1]
	    set prStopY  [lindex $ret 2]
	}
    }
    
    label $fwin.3.s -text "    " -font $fnt
    tixControl $fwin.3.d  -label $_allgText(PrintCalVom) -min 0 -max 31 -integer 1  \
	-options {  entry.width 2 label.width 6 } -variable prStartD -selectmode immediate -autorepeat true
    tixControl $fwin.3.m -options { entry.width 2} -label "." -min 0 -max 12 -integer 1 -variable prStartM -selectmode immediate -autorepeat true
    tixControl $fwin.3.y -options { entry.width 4} -label "." -min 1999 -max 2100 -integer 1  -variable prStartY -selectmode immediate -autorepeat true
    $fwin.3.d subwidget label configure -font $bfnt
    $fwin.3.d subwidget entry configure -font $fnt
    $fwin.3.m subwidget label configure -font $bfnt
    $fwin.3.m subwidget entry configure -font $fnt
    $fwin.3.y subwidget label configure -font $bfnt
    $fwin.3.y subwidget entry configure -font $fnt
    $fwin.3.d  configure -state disabled
    $fwin.3.m  configure -state disabled
    $fwin.3.y  configure -state disabled
    pack $fwin.3.y $fwin.3.m $fwin.3.d $fwin.3.s -side right
    
    label $fwin.4.s -text "    " -font $fnt
    tixControl $fwin.4.d  -label $_allgText(PrintCalBis) -min 0 -max 31 -integer 1  \
	-options {  entry.width 2 label.width 6 } -variable prStopD -selectmode immediate -autorepeat true
    tixControl $fwin.4.m -options { entry.width 2} -label "." -min 0 -max 12 -integer 1 -variable prStopM -selectmode immediate -autorepeat true
    tixControl $fwin.4.y -options { entry.width 4} -label "." -min 1999 -max 2100 -integer 1  -variable prStopY -selectmode immediate -autorepeat true
    $fwin.4.d subwidget label configure -font $bfnt
    $fwin.4.d subwidget entry configure -font $fnt
    $fwin.4.m subwidget label configure -font $bfnt
    $fwin.4.m subwidget entry configure -font $fnt
    $fwin.4.y subwidget label configure -font $bfnt
    $fwin.4.y subwidget entry configure -font $fnt
    $fwin.4.d  configure -state disabled
    $fwin.4.m  configure -state disabled
    $fwin.4.y  configure -state disabled
    pack $fwin.4.y $fwin.4.m $fwin.4.d $fwin.4.s -side right


    checkbutton $fwin.5.c -text "        $_allgText(PrintCalUseColor)" -width 21 -justify left  -anchor nw -state disabled \
	-variable pw_useColor -onvalue color -offvalue graymap  -font $bfnt
    pack $fwin.5.c -side right 

#    canvas  .pr.c.c -width 580 -Aeight 866 -background white
}

proc togglePRdates { } {
    global printMode printCalFormatWin printCalOptionWin
    global printPageFormats printPageFormat


    $printCalFormatWin.1.d configure -state disabled
    $printCalFormatWin.1.m configure -state disabled
    $printCalFormatWin.1.y configure -state disabled
    $printCalFormatWin.2.m configure -state disabled
    $printCalFormatWin.2.y configure -state disabled
    $printCalFormatWin.3.d configure -state disabled
    $printCalFormatWin.3.m configure -state disabled
    $printCalFormatWin.3.y configure -state disabled
    $printCalFormatWin.4.d configure -state disabled
    $printCalFormatWin.4.m configure -state disabled
    $printCalFormatWin.4.y configure -state disabled
    $printCalFormatWin.5.c configure -state disabled


    $printCalOptionWin.2.t configure -state disabled
    $printCalOptionWin.3.i configure -state disabled
    $printCalOptionWin.4.i configure -state disabled
    

    if {$printMode  == "w"} {
	$printCalFormatWin.1.d configure -state normal
	$printCalFormatWin.1.m configure -state normal
	$printCalFormatWin.1.y configure -state normal
	$printCalOptionWin.2.t configure -state normal
	$printCalOptionWin.3.i configure -state normal
    } elseif  {$printMode  == "m"} {
	$printCalFormatWin.2.m configure -state normal
	$printCalFormatWin.2.y configure -state normal
	$printCalOptionWin.2.t configure -state normal
	$printCalOptionWin.3.i configure -state normal
	$printCalOptionWin.4.i configure -state normal
    } elseif  {$printMode  == "l"} {
	$printCalFormatWin.3.d configure -state normal
	$printCalFormatWin.3.m configure -state normal
	$printCalFormatWin.3.y configure -state normal
	$printCalFormatWin.4.d configure -state normal
	$printCalFormatWin.4.m configure -state normal
	$printCalFormatWin.4.y configure -state normal
	$printCalOptionWin.3.i configure -state normal
	$printCalOptionWin.4.i configure -state normal
    } 

    if { $printMode == "c" } { 
	$printCalFormatWin.5.c configure -state  normal
	foreach ent [array names printPageFormats ] {
	    set land [string range $ent 2 2]
	    if { $land == "P" } {
		.pr.t.1.p entryconfigure $ent -state disabled
	    }
	}
	set printPageFormat "[string range $printPageFormat 0 1]L"
    } else {
	foreach ent [array names printPageFormats ] {
	    .pr.t.1.p entryconfigure $ent -state normal
	}
    }
}

proc printCalOptionsDialog { win } {
    global bfnt fnt12
    global pw_ignoInit pw_partLines
    global pw_skipWeekend printCalOptionWin
    global _allgText _buttonText


    pack $win -side top -fill x -pady 2  -padx 2

    set fwin [$win subwidget frame]
    set printCalOptionWin $fwin
    frame $fwin.1 
    frame $fwin.2
    frame $fwin.3
    frame $fwin.4
    
    pack $fwin.1  $fwin.2 $fwin.3  $fwin.4 -side top -fill x -padx 3 -pady 3
    
    tixControl $fwin.2.t  -label $_allgText(PrintLinesPerList)  -min 0 -max 99 -integer 1  -labelside right \
	-options {  entry.width 2  } -variable pw_partLines -selectmode immediate -autorepeat true
    $fwin.2.t subwidget entry configure -font $bfnt
    checkbutton $fwin.3.i -text $_allgText(PrintIgnoreInit) -variable pw_ignoInit -font $bfnt
    checkbutton $fwin.4.i -text $_allgText(PrintIgnoreWeekEnd) -variable pw_skipWeekend -font $bfnt
    pack $fwin.2.t  $fwin.3.i $fwin.4.i -side left -padx 10

    togglePRdates
}

proc printCal { file  } {
    global can selPrinter cputype printMode  env printFile
    global canWidth canHeight _printText
    global printColorMap pw_useColor
    global meterCanceled

    puts "array    : [array exists _printText]"
    puts "elements : [array name _printText]"

    openMeterWithCancel $_printText(MeterTitle) $_printText(MeterPrepare)  1 .pr
    if {$printMode == "c" } {	
	array unset printColorMap
	#normal  appointment
	set printColorMap(\#F8FFF8) "1.0 1.0 1.0 setrgbcolor"
	#info    appointment
	set printColorMap(\#EDEDED) "0.9 0.9 0.9 setrgbcolor"
	#private appointment
	set printColorMap(\#FFE8E0) "0.85 0.85 0.85 setrgbcolor"
	#normal calendar fullday
	set printColorMap(grey80) "0.98 0.98 0.98 setrgbcolor"
	#today calendar fullday
	set printColorMap(PeachPuff) "0.95 0.95 0.95 setrgbcolor"
	#normal calendar 
	set printColorMap(grey95) "1.0 1.0 1.0 setrgbcolor"
	#today calendar 
	set printColorMap(PapayaWhip) "0.98 0.98 0.98 setrgbcolor"
	#Holiday title
	set printColorMap(DarkOliveGreen3) "0.8 0.8 0.8 setrgbcolor"
	#today title
	set printColorMap(ivory) "0.8 0.8 0.8 setrgbcolor"
	#today title text
	set printColorMap(red4) "0.0 0.0 0.0 setrgbcolor"
	#normal title
	set printColorMap(grey98) "1.0 1.0 1.0 setrgbcolor"
	stdCanvas2PS $can $file $canWidth $canHeight $pw_useColor
    } elseif {$printMode == "w" } {
	printWeekCal $file
    } elseif {$printMode == "m" } {
	printMonthCal $file
    } else  {
	printCalList .pr.c.c $file
    }

    if { $meterCanceled } {
	update
	if { [winfo exists .pr] } { destroy .pr}
	return
    }
    set ret [catch {set host $env(HOST)} ]
    if { $ret != 0 } { set host "???" }
    send2Printer $file
	

    closeMeter
}


#WochenKalender

proc printWeekCal {file} {
    global groupOnlyCommon showIgnore ignoreInfo selPrinter
    global username cur_Cal
    global pwDay pwMon pwYear 
    global pwvCan
    global pw_ignoInit pw_printNoPart
    global printCellX printCellY printCellR printCellB printCellC printCellW printCellH
    global meterCanceled _printText  _allgText

    set shift  [ weekDayByDate $pwDay $pwMon $pwYear]
    set ret [calcDate $pwDay $pwMon $pwYear [expr -1* $shift]]
    set pwDay  [lindex $ret 0]
    set pwMon  [lindex $ret 1]
    set pwYear [lindex $ret 2]
    set first [format "%04d%02d%02d"  $pwYear $pwMon $pwDay]
    set last [calcDate2dstr $first 6]


    set fid [open $file "w"]
    set fileName [file rootname $file ]

    set calList [request getUserCal $username  $cur_Cal $first $last  $showIgnore $ignoreInfo [expr 1 - $groupOnlyCommon]]
    if { $calList == "Error" } { return}
    #FIXME Sortierung nach Proritt und Zeit
    set calList [lsort -command compareCalEntries $calList]

    openPWView .pr.c.c $cur_Cal $first $last
    updateMeter 5  $_printText(MeterGetData)
    if { $meterCanceled } {
	destroy .pr 
	return
    }
    set dstr  $first
    for { set d 0} { $d < 7 } {incr d} {
	set day($dstr) $d
	set dstr [calcDate2dstr $dstr  1]
    }
    set anz 0
    set allDatesList ""
    foreach date $calList { 
	set startD [lindex $date 0]
	set stopD [lindex $date 2]
	set did    [lindex $date 6]

	lappend allDatesList "$did $startD"
	if { $startD ==$stopD} {
	    set wd $day($startD)
	    lappend dateList($wd) "$did-$startD"
	    set datum($wd) $startD
	    incr anz
	} else {
	    set thisDate $startD
	    while { $thisDate <= $stopD } {
		set wd $day($thisDate)
		lappend dateList($wd) "$did-$startD"
		set datum($wd) $thisDate
		incr anz
		set thisDate [calcDate2dstr $thisDate 1]
		if { $thisDate > $last } {
		    break
		}
	    }
	}
    } 

    set dc 0
    set list  [eval  request getMultiBriefDate $username [join $allDatesList ] ]
    if { $list == "Error" } { return}

    foreach ent $list {
	set did [lindex $ent 0]
	set startD [lindex $ent 2]
	set dateInfo($did-$startD) $ent
    }
    updateMeter 20  $_printText(MeterCreate)
    if { $meterCanceled } {
	destroy .pr 
	return
    }

    foreach wd [ array names dateList] {
	set isBlocked 0
	set y0 $printCellC($wd)
	
	set pw_printNoPart 0
	foreach did $dateList($wd) {
	    updateMeter [expr int(floor(double($dc)/$anz*72)+20) ] $_printText(MeterCreate)
	    if { $meterCanceled } {
		destroy .pr 
		return
	    }
	    incr dc
	    set ent $dateInfo($did)

	    set titel [lindex $ent 1]
	    set startD [lindex $ent 2]
	    set startT [lindex $ent 3]
	    set stopD  [lindex $ent 4]
	    set stopT  [lindex $ent 5]
	    set room  [lindex $ent 6]
	    set part [lindex $ent 10]
	    set noTime [lindex $ent 13]
	    if { $isBlocked == 0 } {
		set einl ""
		set teiln ""
		foreach ent $part {
		    set name  [ lindex $ent 1]
		    set stat  [ lindex $ent 0]
		    set ugType [ lindex $ent 3]
		    if { $ugType == "g" } {   set name "$_allgText(Gruppe) $name"}
		    if { $stat == "I" && $pw_ignoInit == 0 } { set einl "$einl $name," }
		    if { $stat == "E" } { set einl "$einl $name," }
		    if { $stat == "T" || $stat == "A" } { set teiln "$teiln $name," }
		}
		if { $printCellW($wd) < 160 } {
		    set ret [addDate2CanvasNarrow $printCellX($wd) $y0 $printCellR($wd) $printCellB($wd) $datum($wd) $startD $startT $stopD $stopT $titel $room $einl $teiln $noTime ]
		} else {
		    set ret [addDate2Canvas $printCellX($wd) $y0 $printCellR($wd) $printCellB($wd) $datum($wd) $startD $startT $stopD $stopT $titel $room $einl $teiln $noTime ]
		}
		if {$ret<0  } { 
		    set isBlocked 1
		    addMoreDates $x0 $y0
		} else {
		    set y0 $ret
		}
	    }	    
	}
    }
    updateMeter 92 $_printText(MeterPrepare)
    if { $meterCanceled } {
	destroy .pr 
	return
    }
    update
    canvas2PS .pr.c.c $file 0 0 1

    updateMeter 100 $_printText(Meter2Printer)
    if { $meterCanceled } {
	destroy .pr 
	return
    }
    update
    after 250
    update 
    destroy .pwv
}

proc openPWView {cwin who from to} {
    global printPageFormats printPageFormat printBaseFormats  
    global username fullUserNames curYear longProgName
    global pwvCan  cur_Cal
    global weekDays weekDaysAbk
    global printCellX printCellY printCellR printCellB printCellC printCellW printCellH
    global hbfnt lbfnt bfnt fnt8 bfnt12
    global progname minkoVersion _allgText _printText

    set pwvCan $cwin
    set start [date2Str $from]
    set stop [date2Str $to]

    set base    [ lindex $printPageFormats($printPageFormat) 1]
    set cwidth  [ lindex $printBaseFormats($base)   0]
    set cheight [ lindex $printBaseFormats($base)   1]

    set cols 2
    set rows 3
    if { $cheight < $cwidth } {
	set cols 6
	set rows 1
    }
    $cwin configure -width  $cwidth
    $cwin configure -height $cheight



    set xr [expr $cwidth]
    set cellWidth [expr int(floor($cwidth/$cols))]
    for { set c 0 } { $c < $cols} {incr c } {
	set xk($c) [expr $xr - ($cols -$c) * $cellWidth ]
    }
    set x0 $xk(0)
    set xc [expr int(floor( ($xr+$x0)/2))]
    

    set headHeight 25
    set tailHeight 10
    set subCellHeight  [ expr int(floor(($cheight -$headHeight -$tailHeight)/2/$rows)) ]
    set cellHeight [ expr 2*$subCellHeight ]
    set y0 0
    set y1 26
    set yt [expr $y1 + 3* $cellHeight]
    set tailHeight [expr $cheight - $yt -1 ]
    for {set i 0 } { $i <$rows } {incr i} {
	for {set j 0 } { $j <$cols} {incr j} {
	    set c [ expr $cols*$i +$j]
	    set printCellX($c) $xk($j)
	    set printCellY($c) [expr $i*$cellHeight + $y1]
	    set printCellW($c) $cellWidth
	    if {$c <5 } {
		set printCellH($c) $cellHeight
	    } else {
		set printCellH($c) $subCellHeight
	    }
	}
    }
    set printCellX(6) $printCellX(5)
    set printCellY(6) [expr $printCellY(5) +$subCellHeight] 
    set printCellW(6) $printCellW(5)
    set printCellH(6) $printCellH(5)
    foreach c {0 1 2 3 4 5 6} {
	#right border
	set printCellR($c) [expr $printCellX($c) +$printCellW($c)]
	#bottom border
	set printCellB($c) [expr $printCellY($c) +$printCellH($c)]
	#start of appointment array in cell
	set printCellC($c) [expr $printCellY($c) + 18 ]  
    }

    set fwho [calUserList2NameList $cur_Cal]

    $pwvCan  create rectangle $x0 $y0 $xr $y1 -fill grey80 -outline black
    set txt "$_printText(CalWeek1) $fwho $_printText(CalWeek2) $_printText(CalFrom) $start $_printText(CalTo) $stop"
    if { [font measure $lbfnt $txt] > 608 } {
	set txt "$_printText(CalWeek1) $fwho _printText(CalWeek2)"
	$cwin create text $xc 2 -text $txt -justify center -anchor n -font $bfnt12
	
	set txt [strTrunc4 $_printText(CalWeek1) $fwho $_printText(CalWeek2) [expr $cwidth-8] $bfnt12]
	$cwin create text $xc 14 -text $txt -justify center -anchor n -font $bfnt12
    } else  {
	$cwin create text $xc 5 -text $txt -justify center -anchor n -font $lbfnt
    }
    
    set datum $from
    foreach c {0 1 2 3 4 5 6} {
	$cwin create rectangle $printCellX($c) $printCellY($c) $printCellR($c) $printCellB($c) \
	    -fill white -outline black
	set ye [expr  $printCellC($c) -2]
	$cwin create rectangle $printCellX($c) $printCellY($c) $printCellR($c) $ye \
	    -fill grey90 -outline black
	set txt [format "%s %s %s" [lindex $weekDays $c] $_allgText(dateThe) [ date2Str $datum]]
	if { [font measure $bfnt $txt] > [expr $printCellW($c) -8 ] } {
	    set txt [format "%s; %s" [lindex $weekDays $c] [ date2Str $datum]]
	}
	if { [font measure $bfnt $txt] > [expr $printCellW($c) -8 ] } {
	    set txt [format "%s; %s" [lindex $weekDaysAbk $c] [ date2Str $datum]]
	}
	if { [font measure $bfnt $txt] > [expr $printCellW($c) -8 ] } {
	    set txt [ date2Str $datum]
	}
	$cwin create text  [expr $printCellX($c)+8] [expr $printCellY($c) +4]\
	    -text $txt -justify left -anchor nw -font $bfnt
	set datum [calcDate2dstr $datum 1]
    }
    $cwin create text $xr $yt -text "created by $longProgName $minkoVersion ($curYear)"\
	-justify right -anchor ne -font $fnt8
}


proc printMonthCal {file} {
    global groupOnlyCommon showIgnore ignoreInfo selPrinter
    global username cur_Cal
    global pwDay pwMon pwYear 
    global pwvCan
    global monlen
    global pw_ignoInit
    global pw_printNoPart 
    global pw_skipWeekend
    global printCellX printCellY printCellR printCellB printCellC printCellW printCellH
    global meterCanceled _printText _allgText

    set dayCount [lindex $monlen $pwMon]
    if { [expr $pwYear %4] == 0  && $pwMon == 2} { incr dayCount}
    
    set shift  [ weekDayByDate 1 $pwMon $pwYear]
    set ret [calcDate 1 $pwMon $pwYear [expr -1* $shift]]
    set begDay  [lindex $ret 0]
    set begMon  [lindex $ret 1]
    set begYear [lindex $ret 2]
    set first [format "%04d%02d%02d"  $begYear $begMon $begDay]
    incr dayCount $shift
    
    set endDay [lindex $monlen $pwMon]
    if { [expr $pwYear %4] == 0  && $pwMon == 2} { incr endDay}
    set shift  [expr 6-[ weekDayByDate $endDay $pwMon $pwYear] ]
    set ret [calcDate $endDay $pwMon $pwYear  $shift ]
    set endDay  [lindex $ret 0]
    set endMon  [lindex $ret 1]
    set endYear [lindex $ret 2]
    set last [format "%04d%02d%02d"  $endYear $endMon $endDay]
    incr dayCount $shift

    set weekCount [expr int(floor($dayCount /7)) ]
    set over [expr $dayCount -7*$weekCount]




    set dstr  $first
    for { set w 0 } { $w < $weekCount } {incr  w} { set dayList($w) ""}
    set allDaysList ""
    for { set w 0 } { $w < $weekCount } {incr  w} {
	for { set d 0} { $d < 7 } {incr d} {
	    lappend dayList($w) $dstr 
	    set dstr [calcDate2dstr $dstr  1]
	    set week($w-$d) $w
	    set day($w-$d) $d
	}
	lappend allDaysList $dayList($w) 
    }
    


    set fid [open $file "w"]
    set fileName [file rootname $file ]

    set calList [request getUserCal $username  $cur_Cal $first $last  $showIgnore $ignoreInfo [expr 1 - $groupOnlyCommon]]
    if { $calList == "Error" } { return}

    #FIXME Sortierung nach Proritt und Zeit
    set calList [lsort -command compareCalEntries $calList]

    openPMView .pr.c.c $cur_Cal $first $last $pwMon $weekCount  

    updateMeter 5  $_printText(MeterGetData)
    if { $meterCanceled } {
	destroy .pr 
	return
    }

    set anz 0
    set allDatesList ""
    foreach date $calList { 
	set startD [lindex $date 0]
	set stopD [lindex $date 2]
	set did    [lindex $date 6]

	lappend allDatesList "$did $startD"
	if { $startD ==$stopD} {
	    set wn [lsearch -glob $allDaysList *$startD* ]
	    set wd [lsearch -exact $dayList($wn) $startD ]
	    lappend dateList($wn-$wd) "$did-$startD"
	    set datum($wn-$wd) $startD
	    incr anz
	} else {
	    set thisDate $startD
	    while { $thisDate <= $stopD } {
		set wn [lsearch -glob $allDaysList *$thisDate* ]
		set wd [lsearch -exact $dayList($wn) $thisDate ]
		lappend dateList($wn-$wd) "$did-$startD"
		set datum($wn-$wd) $thisDate
		incr anz
		set thisDate [calcDate2dstr $thisDate 1]
		if { $thisDate > $last } {
		    break
		}
	    }
	}
    } 

    updateMeter 15 $_printText(MeterPrepare)
    if { $meterCanceled } {
	destroy .pr 
	return
    }


    set lstart 0
    set adanz [llength $allDatesList ]
    while { $lstart < $adanz }  {
	set last [ expr $lstart +49 ]
	if { $last > $adanz } { set last "end" }
	set dc 0
	set list  [eval  request getMultiBriefDate $username [join [lrange $allDatesList $lstart $last ] ] ]
	if { $list == "Error" } { return}

	foreach ent $list {
	    set did [lindex $ent 0]
	    set startD [lindex $ent 2]
	    set dateInfo($did-$startD) $ent
	}
	incr lstart 50
    }
    updateMeter 20 $_printText(MeterCreate)
    if { $meterCanceled } {
	destroy .pr 
	return
    }

    foreach idx [ array names dateList] {
	set wd [ lindex [split $idx -] 1]
	if { $pw_skipWeekend == 1} {
	    if { $wd >4 } { continue} 
	}
	set x0 $printCellX($idx)
	set x1 $printCellR($idx)
	set y0 $printCellC($idx)
	set y1 $printCellB($idx)
	set isBlocked 0
	set pw_printNoPart 0
	if { $meterCanceled } {
	    destroy .pr 
	    return
	}
	foreach did $dateList($idx) {
	    updateMeter [expr int(floor(double($dc)/$anz*72)+20) ] $_printText(MeterCreate)
	    if { $meterCanceled } {
		destroy .pr 
		return
	    }
	    incr dc
	    set ent $dateInfo($did)
	    set did   [lindex $ent 0]
	    set titel [lindex $ent 1]
	    set startD [lindex $ent 2]
	    set startT [lindex $ent 3]
	    set stopD  [lindex $ent 4]
	    set stopT  [lindex $ent 5]
	    set room  [lindex $ent 6]
	    set part [lindex $ent 10]
	    set noTime [lindex $ent 13]
	    if { $isBlocked == 0 } {
		set einl ""
		set teiln ""
		foreach ent $part {
		    set name  [ lindex $ent 1]
		    set stat  [ lindex $ent 0]
		    set ugType [ lindex $ent 3]
		    if { $ugType == "g" } {   set name "$_allgText(Gruppe) $name"}
		    if { $stat == "I" && $pw_ignoInit == 0 } { set einl "$einl $name," }
		    if { $stat == "E" } { set einl "$einl $name," }
		    if { $stat == "T" || $stat == "A" } { set teiln "$teiln $name," }
		}
		if { $printCellW($idx) < 250 } {
		    set ret [addDate2CanvasNarrow $x0 $y0 $x1 $y1 $datum($idx) $startD $startT $stopD $stopT $titel $room $einl $teiln $noTime ]
		} else {
		    set ret [addDate2Canvas $x0 $y0 $x1 $y1 $datum($idx) $startD $startT $stopD $stopT $titel $room $einl $teiln $noTime ]
		}
		if {$ret<0  } { 
		    set isBlocked 1
		    addMoreDatesinMonth [expr int(floor(($x0+$x1)/2))] $y0
		} else {
		    set y0 $ret
		}
	    }
	}
    }  

    updateMeter 92 $_printText(MeterPrepare)
    if { $meterCanceled } {
	destroy .pr 
	return
    }
    update
    canvas2PS .pr.c.c $file 0 0 2
    updateMeter 100 $_printText(Meter2Printer)
    if { $meterCanceled } {
	destroy .pr 
	return
    }
    destroy .pwv  

}

proc openPMView {cwin  who from to mon   weekCount  } {
    global printPageFormats printPageFormat printBaseFormats  
    global username fullUserNames curYear longProgName
    global pwvCan  cur_Cal
    global weekDays weekDaysAbk
    global printCellX printCellY printCellR printCellB printCellC printCellW printCellH
    global hbfnt lbfnt bfnt fnt8 bfnt12 bfnt18
    global progname minkoVersion _allgText _printText
    global pw_skipWeekend
    global weekDays monNames

    set pwvCan $cwin
    set start [date2Str $from]
    set stop [date2Str $to]

    set base    [ lindex $printPageFormats($printPageFormat) 1]
    set cwidth  [expr [ lindex $printBaseFormats($base)   0] *2 ]
    set cheight [expr [ lindex $printBaseFormats($base)   1] *2 ]


    if {$pw_skipWeekend == 1 } {
	set cols 5
	set wds  5
    } else {
	set cols 6
	set wds 7
    }
    set rows $weekCount
    $cwin configure -width  $cwidth
    $cwin configure -height $cheight



    set xr [expr $cwidth-1]
    set cellWidth [expr int(floor($cwidth/$cols))]
    for { set c 0 } { $c < $cols} {incr c } {
	set xk($c) [expr $xr - ($cols -$c) * $cellWidth +1]
    }
    set x0 $xk(0)
    set xc [expr int(floor( ($xr+$x0)/2))]
    

    set headHeight 43
    set tailHeight 16
    set subCellHeight  [ expr int(floor(($cheight -$headHeight -$tailHeight)/2/$rows)) ]
    set cellHeight [ expr 2*$subCellHeight ]
    set y0 0
    set y1 47
    set yt [expr $y1 + 3* $cellHeight]
    set tailHeight [expr $cheight - $yt -1 ]
    for {set w 0 } { $w <$rows } {incr w} {
	for {set d 0 } { $d <$cols} {incr d} {
	    set c "$w-$d"
	    set printCellX($c) $xk($d)
	    set printCellY($c) [expr $w*$cellHeight + $y1]
	    set printCellW($c) $cellWidth
	    if { $d<5} {
		set printCellH($c) $cellHeight
	    } else {
		set printCellH($c) $subCellHeight
	    }
	    #right border
	    set printCellR($c) [expr $printCellX($c) +$printCellW($c)]
	    #bottom border
	    set printCellB($c) [expr $printCellY($c) +$printCellH($c)]
	    #start of appointment array in cell
	    set printCellC($c) [expr $printCellY($c) + 18 ]  
	    
	}
	if { $pw_skipWeekend ==0 } {
	    set c5 "$w-5"
	    set c6 "$w-6"
	    set printCellX($c6) $printCellX($c5)
	    set printCellY($c6) [expr $printCellY($c5)+$subCellHeight]
	    set printCellW($c6) $printCellW($c5)
	    set printCellH($c6) $printCellH($c5)
	    #right border
	    set printCellR($c6) [expr $printCellX($c6) +$printCellW($c6)]
	    #bottom border
	    set printCellB($c6) [expr $printCellY($c6) +$printCellH($c6)]
	    #start of appointment array in cell
	    set printCellC($c6) [expr $printCellY($c6) + 18 ] 
	}
    }


    set fwho [calUserList2NameList $cur_Cal]

    $pwvCan  create rectangle $x0 $y0 $xr $y1 -fill grey80 -outline black
#    set txt "Monatskalender [lindex $monNames $mon] fr $fwho "
    $cwin create text $xc 5 -text "$_printText(CalMonth1) [lindex $monNames $mon] $_printText(CalMonth2)" \
	-justify center -anchor n -font $bfnt18
    set txt [strTrunc3 $fwho [expr $cwidth -8] $bfnt18 [string length $fwho] ]
    $cwin create text $xc 25 -text $txt -justify center -anchor n -font $bfnt18
    
    set datum $from
    for {set w 0 } { $w <$rows } {incr w} {
	for {set d 0 } { $d <$wds} {incr d} {
	    set c "$w-$d"
	    $cwin create rectangle $printCellX($c) $printCellY($c) $printCellR($c) $printCellB($c) \
		-fill white -outline black
	    set ye [expr  $printCellC($c) -2]
	    $cwin create rectangle $printCellX($c) $printCellY($c) $printCellR($c) $ye \
		-fill grey90 -outline black
	    set txt [format "%s %s %s" [lindex $weekDays $d] $_allgText(dateThe) [ date2Str $datum]]
	    if { [font measure $bfnt $txt] > [expr $printCellW($c) -8 ] } {
		set txt [format "%s; %s" [lindex $weekDays $d] [ date2Str $datum]]
	    }
	    if { [font measure $bfnt $txt] > [expr $printCellW($c) -8 ] } {
		set txt [format "%s; %s" [lindex $weekDaysAbk $d] [ date2Str $datum]]
	    }
	    if { [font measure $bfnt $txt] > [expr $printCellW($c) -8 ] } {
		set txt [ date2Str $datum]
	    }
	    $cwin create text  [expr $printCellX($c)+8] [expr $printCellY($c) +4]\
		-text $txt -justify left -anchor nw -font $bfnt
	    set datum [calcDate2dstr $datum 1]
	}
    }
    $cwin create text $xr $yt -text "created by $longProgName $minkoVersion ($curYear)"\
	-justify right -anchor ne -font $fnt8
}





proc addDate2Canvas { x0 y0 xmax ymax thisDate startD start stopD stop title room einl teiln noTime} {
    global pwvCan amdy
    global bfnt10 fnt10 bfnt8 fnt8 pfnt8 pbfnt8 fnt7
    global pw_printNoPart pw_partLines _calText

    set startTime [ time2Str $start]
    set stopTime  [ time2Str $stop]
    
    set x1 [ expr $x0 +2]
    set x1b [ expr $x0 +2]
    set x2 [ expr $x0 +30]
    set x3 [ expr $x0 +35]
    set x4 [ expr $x0 +60]
    set x5 [ expr $x0 +105]
    set x6 [ expr $x0 +105]
    set y1 [ expr $y0 +0 ]
    set y2 [ expr $y0 +10]
    set y3 [ expr $y0 +19]
    set y4 [ expr $y0 +19]

    set einl [string trim $einl ", "]
    set list(0) [string trim "$einl; $teiln" " ," ]
    set yl(0) $y4
    set lanz 0
    set l -1

    if {$pw_partLines ==0 } {set pw_printNoPart 1 }
    if { $pw_printNoPart == 0} {
	set dc ""
	set lw [ expr ($xmax - $x0) - 105 -4]
	set lastLine [ expr $pw_partLines -1]
	while { [string length $list($lanz) ] > 0 } {
	    incr lanz
	    incr l
	    set list($lanz) ""
	    set list($l) [string trim $list($l) " ,;" ]
	    set dc ","
	    set inName 0
	    while { [ font measure $fnt7 $list($l) ] > $lw } {
		set p1 [string last "," $list($l) ]
		set p2 [string last ";" $list($l) ]
		set p3 [string last " " $list($l) ]
		if { $p1 > $p2 } {
		    set p $p1
		    set dc ","
		} else  {
		    set p $p2
		    set dc ";"
		}
		if { $p3 > $p } {
		    set p $p3
		    set dc " "
		}
		set txt [string trim [string trim [ string range $list($l)  $p end ] " "] "," ]
		if { [string length $list($lanz) ] > 0 } { 
		    if { $dc != " " } {
			set list($lanz) "$dc$list($lanz)"
			set inName 0
		    } else {
			set inName 1
			set list($lanz) " $txt$list($lanz)"
		    }
		} else {
		    set list($lanz) "$dc$txt"
		}
		incr p -1
		set list($l) [string range $list($l) 0 $p ]
	    }
	    #	set lw 170
	    if { [string length $list($lanz) ] > 0 || $dc == ";"} {
		set list($l) "$list($l)$dc"
	    }
	    if {$l == $lastLine } {
		if { $inName == 1} {
		    set p [string last "," $list($l) ]
		    set p2 [string last ";" $list($l) ]
		    if { $p2 > $p } {set p $p2}
		    set list($l)  [string range $list($l) 0 $p ]
		}
		set list($l) "$list($l) ..."
		if { [ font measure $fnt7 $list($l) ] > $lw } {
		    set p [string last "," $list($l) ]
		    set p2 [string last ";" $list($l) ]
		    if { $p2 > $p } {set p $p2}
		    set list($l)  [string range $list($l) 0 $p ]
		    set p [string last "," $list($l) ]
		    set p2 [string last ";" $list($l) ]
		    if { $p2 > $p } {set p $p2}
		    set list($l)  [string range $list($l) 0 $p ]
		    set list($l) "$list($l) ..."
		}

		break
	    }
	}
    }
    if { $lanz >1 } {
 	for { set l 1} {$l <$lanz } {incr l } {
	    set lp [expr $l -1]
 	    set yl($l) [expr $yl($lp) +8  ]
	    set ylm $yl($l)
 	}
    } else {
	set ylm $yl(0)
    }
    if { $pw_printNoPart == 1 } {
	set ylm [expr $y2+8]
    } else {
	incr ylm 9
    }
    if { $ylm > $ymax } {
	if { $pw_printNoPart == 0 } {
	    set pw_printNoPart 1
	    set ylm [expr $y2+9]
	    if { $ylm > $ymax } {return -1}
	} else  {
	    return -1
	}
    }

    if { $noTime == 1} {
	    $pwvCan create text $x1 $y1 -text $_calText(FullDay) -font $fnt10 -justify left -anchor nw
    } else {
	if { $thisDate != $startD } {
	    if  {$thisDate != $stopD } {
		set startDate  [ date2Str $startD]
		set stopDate  [ date2Str $stopD]
		$pwvCan create text $x1  $y1 -text "$startDate," -font $fnt8 -justify left -anchor nw
		$pwvCan create text $x3  $y1 -text "$startTime" -font $fnt8 -justify left -anchor nw
		$pwvCan create text $x2  $y2 -text "-"       -font $fnt8 -justify left -anchor nw
		$pwvCan create text $x1  $y3 -text "$stopDate," -font $fnt8 -justify left -anchor nw
		$pwvCan create text $x3  $y3 -text "$stopTime" -font $fnt8 -justify left -anchor nw
	    } else  {
		set startDate  [ date2Str $startD]
		set stopDate  [ date2Str $stopD]
		$pwvCan create text $x1  $y1 -text "$startDate," -font $fnt8 -justify left -anchor nw
		$pwvCan create text $x3  $y1 -text "$startTime" -font $fnt8 -justify left -anchor nw
		$pwvCan create text $x2  $y2 -text "-"       -font $fnt8 -justify left -anchor nw
		$pwvCan create text $x3  $y2 -text "$stopTime" -font $fnt8 -justify left -anchor nw
	    }
	} else {
	    if { $startD == $stopD } {
		$pwvCan create text $x1 $y1 -text $startTime  -font $fnt8 -justify left -anchor nw
		$pwvCan create text $x2 $y1 -text "-"       -font $fnt8 -justify left -anchor nw
		$pwvCan create text $x3 $y1 -text $stopTime   -font $fnt8 -justify left -anchor nw
	    } else {
		set stopDate  [ date2Str $stopD]
		$pwvCan create text $x1  $y1 -text $startTime -font $fnt8 -justify left -anchor nw
		$pwvCan create text $x2  $y1 -text "-"       -font $fnt8 -justify left -anchor nw
		$pwvCan create text $x1  $y2 -text "$stopDate," -font $fnt8 -justify left -anchor nw
		$pwvCan create text $x3  $y2 -text "$stopTime" -font $fnt8 -justify left -anchor nw
		#	    $pwvCan create text $x3 $y3 -text $stopTime -font $fnt8 -justify left -anchor nw
	    }
	}
    }
    set titel [strTrunc3 $title [expr $xmax -$x4] $bfnt10 [string length $title] ]
    $pwvCan create text $x4 $y1 -text $titel      -font $bfnt10 -justify left -anchor nw

    $pwvCan create text $x4 $y2 -text "Ort"          -font $fnt8 -justify left -anchor nw
    $pwvCan create text $x5 $y2 -text "$room"        -font $fnt8 -justify left -anchor nw
    if { $pw_printNoPart == 0 } {
	$pwvCan create text $x4 $y4 -text "Teilnehmer"   -font $fnt7 -justify left -anchor nw
	$pwvCan create text $x5 $yl(0) -text $list(0) -font $fnt7 -justify left -anchor nw
	for { set l 1} {$l <$lanz } {incr l } {
	    $pwvCan create text $x6 $yl($l) -text $list($l) -font $fnt7 -justify left -anchor nw
	}
    }
    return $ylm
}
proc addMoreDates { x0 y0} {
    global pwvCan  bfnt10
    
    $pwvCan create text [expr $x0+145] [expr $y0 +2] -text "... weitere Termine ... " -font $bfnt10 -justify center -anchor n

}

proc addDate2CanvasNarrow { x0 y0 xmax ymax thisDate startD start stopD stop title room einl teiln noTime} {
    global pwvCan amdy
    global bfnt8 fnt8 bfnt6 fnt6 pfnt6 pbfnt6 fnt10 bfnt10 fnt12 bfnt12 bfnt11
    global pw_printNoPart pw_partLines
    global pw_skipWeekend

    set startTime [ time2Str $start]
    set stopTime  [ time2Str $stop]
    
    set x1 [ expr $x0 +6]
    set x2 [ expr $x0 +52]
    set x3 [ expr $x0 +60]
    set x2b [ expr $x0 +72]
    set x3b [ expr $x0 +80]
    set x2c [ expr $xmax -76]
    set x3c [ expr $xmax -68]
    if { $pw_skipWeekend == 1} {
	set x2b [ expr $x0 +82]
	set x3b [ expr $x0 +90]
	set x2c [ expr $xmax -96]
	set x3c [ expr $xmax -88]
    }

    set xt [ expr $x0 +2 ]
    set xl [ expr $x0 +6 ]
    set xc [ expr $x0 +60]
    set y1 [ expr $y0 +0 ]
    set y2 [ expr $y0 +10]
    set y3 [ expr $y0 +19]
    set y4 [ expr $y0 +28]
    
    set einl [string trim $einl ", "]
    set list(0) [string trim "$einl, $teiln" " ," ]
    set yl(0) $y4
    set lanz 0
    set l -1

    set width [expr ($xmax - $x0)]
    set lw [ expr $width  - 58  ]

    if {$pw_partLines ==0 } {set pw_printNoPart 1 }
    if { $pw_printNoPart == 0} {
	set dc ""
	set lastLine [ expr $pw_partLines -1]
	while { [string length $list($lanz) ] > 0 } {
	    incr lanz
	    incr l
	    set list($lanz) ""
	    set list($l) [string trim $list($l) " ,;" ]
	    set dc ","
	    set inName 0
	    while { [ font measure $fnt10 $list($l) ] > $lw } {
		set p1 [string last "," $list($l) ]
		set p2 [string last ";" $list($l) ]
		set p3 [string last " " $list($l) ]
		if { $p1 > $p2 } {
		    set p $p1
		    set dc ","
		} else  {
		    set p $p2
		    set dc ";"
		}
		if { $p3 > $p } {
		    set p $p3
		    set dc " "
		}
		set txt [string trim [string trim [ string range $list($l)  $p end ] " "] "," ]
		if { [string length $list($lanz) ] > 0 } { 
		    if { $dc != " " } {
			set list($lanz) "$dc$list($lanz)"
			set inName 0
		    } else {
			set inName 1
			set list($lanz) " $txt$list($lanz)"
		    }
		} else {
		    #		if { $dc == ";" } {
		    set list($lanz) "$dc$txt"
		    #		} else { 
		    #		    set list($lanz) $txt
		    #		}
		}
		incr p -1
		set list($l) [string range $list($l) 0 $p ]
	    }
	    if { [string length $list($lanz) ] > 0 || $dc == ";"} {
		set list($l) "$list($l)$dc"
	    }
	    if {$l == $lastLine } {
		if { $inName == 1} {
		    set p [string last "," $list($l) ]
		    set p2 [string last ";" $list($l) ]
		    if { $p2 > $p } {set p $p2}
		    set list($l)  [string range $list($l) 0 $p ]
		}
		set list($l) "$list($l) ..."
		if { [ font measure $fnt10 $list($l) ] > $lw } {
		    set p [string last "," $list($l) ]
		    set p2 [string last ";" $list($l) ]
		    if { $p2 > $p } {set p $p2}
		    set list($l)  [string range $list($l) 0 $p ]
		    set p [string last "," $list($l) ]
		    set p2 [string last ";" $list($l) ]
		    if { $p2 > $p } {set p $p2}
		    set list($l)  [string range $list($l) 0 $p ]
		    set list($l) "$list($l) ..."
		}

		break
	    }
	}
    }

    if { $lanz >1 } {
 	for { set l 1} {$l <$lanz } {incr l } {
	    set lp [expr $l -1]
 	    set yl($l) [expr $yl($lp) +9  ]
	    set ylm $yl($l)
 	}
    } else {
	set ylm $yl(0)
    }

    if { $pw_printNoPart == 1 } {
	set ylm [expr $y3+8]
    } else {
	incr ylm 11
    }
    if { $ylm > $ymax } {
	if { $pw_printNoPart == 0 } {
	    set pw_printNoPart 1
	    set ylm [expr $y3+9]
	    if { $ylm > $ymax } {return -1}
	} else  {
	    return -1
	}
    }

    if { $noTime == 1} {
	    $pwvCan create text $x1 $y2 -text "ganztgig" -font $fnt10 -justify left -anchor nw
    } else {
	if { $thisDate != $startD } {
	    if  {$thisDate != $stopD } {
		set startDate  [ date2Str $startD]
		set stopDate  [ date2Str $stopD]
		$pwvCan create text $x1  $y2 -text "$startDate, $startTime" -font $fnt10 -justify left -anchor nw
		$pwvCan create text $x2c $y2 -text "-"       -font $fnt10 -justify left -anchor nw
		$pwvCan create text $x3c $y2 -text "$stopDate, $stopTime" -font $fnt10 -justify left -anchor nw
	    } else  {
		set startDate  [ date2Str $startD]
		set stopDate  [ date2Str $stopD]
		$pwvCan create text $x1  $y2 -text "$startDate, $startTime" -font $fnt10 -justify left -anchor nw
		$pwvCan create text $x2b $y2 -text "-"       -font $fnt10 -justify left -anchor nw
		$pwvCan create text $x3b $y2 -text "$stopTime" -font $fnt10 -justify left -anchor nw
	    }
	} else {
	    if { $startD == $stopD } {
		$pwvCan create text $x1 $y2 -text $startTime  -font $fnt10 -justify left -anchor nw
		$pwvCan create text $x2 $y2 -text "-"       -font $fnt10 -justify left -anchor nw
		$pwvCan create text $x3 $y2 -text $stopTime   -font $fnt10 -justify left -anchor nw
	    } else {
		set stopDate  [ date2Str $stopD]
		$pwvCan create text $x1 $y2 -text $startTime -font $fnt10 -justify left -anchor nw
		$pwvCan create text $x2c $y2 -text "-"       -font $fnt10 -justify left -anchor nw
		$pwvCan create text $x3c $y2 -text "$stopDate, $stopTime" -font $fnt10 -justify left -anchor nw
	    }
	}
    }
    set titel [strTrunc3 $title [expr $width - 12] $bfnt11 [string length $title] ]
    $pwvCan create text $xt $y1 -text $titel  -font $bfnt11 -justify left -anchor nw

    set raum $room
    set len [ string length $room]
    set tl [font measure $fnt10 $raum ]
    while { $tl > $lw } { 
	incr len -1
	set raum "[ string range $room 0 $len] ..."
	set tl [font measure $fnt10 $raum ]
    }
    $pwvCan create text $xl $y3 -text "Ort"          -font $fnt10 -justify left -anchor nw
    $pwvCan create text $xc $y3 -text "$raum"        -font $fnt10 -justify left -anchor nw
    if { $pw_printNoPart == 0 } {
	$pwvCan create text $xl $y4 -text "Teilnehmer"   -font $fnt10 -justify left -anchor nw
	$pwvCan create text $xc $yl(0) -text $list(0) -font $fnt10 -justify left -anchor nw
	for { set l 1} {$l <$lanz } {incr l } {
	    $pwvCan create text $xc $yl($l) -text $list($l) -font $fnt10 -justify left -anchor nw
	}
    }
    return $ylm
}
proc addMoreDatesinMonth { x0 y0} {
    global pwvCan  bfnt8 bfnt10
    
    $pwvCan create text [expr $x0] [expr $y0+2] -text "... weitere Termine ... " -font $bfnt10 -justify center -anchor n

}


#Liste


proc printCalList {cwin bfile} {
    global cur_Cal groupOnlyCommon username showIgnore ignoreInfo
    global bfnt12 fnt12 fnt10
    global userList groupList roomList partStatusStr
    global prStartD prStartM prStartY prStopD prStopM prStopY
    global pw_ignoInit
    global pw_skipWeekend
    global meterCanceled _printText
    global printPageFormats printPageFormat printBaseFormats  
    global _printText _allgText _calText

    set base    [ lindex $printPageFormats($printPageFormat) 1]
    set cwidth  [ lindex $printBaseFormats($base)   0]
    set cheight [ lindex $printBaseFormats($base)   1]
    
    $cwin configure -width  $cwidth
    $cwin configure -height $cheight
    

    set first [format "%04d%02d%02d" $prStartY $prStartM $prStartD]
    set last  [format "%04d%02d%02d" $prStopY $prStopM $prStopD]
    updateMeter 5  $_printText(MeterGetData)
    if { $meterCanceled } {
	destroy .pr 
	return
    }
    set calList [request getUserCal $username  $cur_Cal $first $last  $showIgnore $ignoreInfo [expr 1 - $groupOnlyCommon]]
    if { $calList == "Error" } { return}


    set xr 0
    set x1 [expr $xr +6] 
    set x2 [expr $xr +104]
    set x3 [expr $xr +120] 
    set x4 [expr $xr +180]
    set twidth [expr $cwidth -$x3 -4]

    set file "$bfile.tmp"
    set fid [open $bfile "w"]

    set fc 0
    set page 1
    set dy 14
    set dy2 4
    set dy3 12
    set ymax [expr $cheight -(2+$dy +3*$dy3)]
    set lc 0
    set calList [lsort -command compareCalEntries2 $calList]
    set anz [llength $calList]
    set allDatesList ""
    foreach date $calList { 
	set startD [lindex $date 0]
	set did    [lindex $date 6]
	lappend allDatesList "$did $startD"
    }

    set ystart [drawListPrintPageHead $cwin 1 $cwidth $cheight ]
    set y $ystart

    set dateList [eval  request getMultiBriefDate $username [join $allDatesList ] ]
    if { $dateList == "Error" } { return}


    set psText ""
    set anz [llength $dateList]
    set count 0
    foreach ent $dateList { 
	updateMeter [expr int(floor(double($lc)/$anz*100)) ] $_printText(MeterCreate)
	if { $meterCanceled } {
	    destroy .pr 
	    return
	}
	set titel [lindex $ent 1]
	set startD [lindex $ent 2]
	set startT [lindex $ent 3]
	set stopD  [lindex $ent 4]
	set stopT  [lindex $ent 5]
	set room  [lindex $ent 6]
	set details [lindex $ent 7]
	set priv ""
 	if { [lindex $ent 9] == 1} { set priv $_printText(CalAppGroupOnly)}
 	if { [lindex $ent 8] == 1} { set priv $_printText(CalAppPrivate)}
	set part [lindex $ent 10]
 	set group [lindex $ent 11]
 	set repTyp [lindex $ent 12]
	set noTime [lindex $ent 13]
	


	set startWD [weekDayByDateStr $startD]
	if { $startD == $stopD} {
	    if { $startWD >4} {
		continue
	    }
	}
 	
 	set m0 [expr $startT %100]
 	set h0 [expr ($startT -$m0 ) /100 ]
 	set m1 [expr $stopT %100]
 	set h1 [expr ($stopT -$m0 ) /100 ]
 	set uList ""
 	foreach ent $part {
 	    set name  [ lindex $ent 1]
 	    set stat  [ lindex $ent 2]
 	    set ugType [ lindex $ent 3]
	    if { [lindex $ent 0] != "I" || $pw_ignoInit == 0 } {
		set uList "$uList, $name"
	    }
 	}
	set ulist [string trim $uList ", "]
	incr y 2

	set yl [expr $y -6]
 	$cwin create line 0 $yl $cwidth $yl
	if {$noTime == 1} {
	    $cwin create text $x1 $y -font $fnt12 -text [format "  %s" [date2lstr $startD] ] -anchor nw
	    if { $startD != $stopD } {
		$cwin create text $x1 $y2 -font $fnt12 -text [format "- %s" [date2lstr $stopD] ] -anchor nw
	    }
	} else {
	    #+1 to fix font size difference on print
	    set y2 [expr $y +$dy3 +1] 
	    $cwin create text $x1 $y -font $fnt12 -text "[date2lstr $startD]," -anchor nw 
	    $cwin create text $x2 $y -font $fnt12 -text [format "%2d:%02d" $h0 $m0] -anchor ne -justify right
	    if { $startD != $stopD } {
		$cwin create text $x1 $y2 -font $fnt12 -text "[date2lstr $stopD]," -anchor nw 
	    }
	    $cwin create text $x2 $y2 -font $fnt12 -text [format "%2d:%02d" $h1 $m1] -anchor ne -justify right
	}
	set txt [strTrunc3  $titel $twidth $bfnt12 [string length $titel] ]
 	$cwin create text $x4 $y -font $bfnt12  -text  $txt -anchor nw

 	incr y $dy
 	$cwin create text $x3 $y -font $fnt10 -text "$_allgText(Ort): " -anchor nw
 	$cwin create text $x4 $y -font $fnt10 -text $room    -width 320 -anchor nw 
 	incr y $dy3
 	$cwin create text $x3 $y -font $fnt10 -text "$_calText(Details): " -anchor nw
 	$cwin create text $x4 $y -font $fnt10 -text $details -width 320 -anchor nw -tag o$lc
	set yl [ lindex [ $cwin bbox o$lc ] 3]
	while {$y< $yl } {incr y $dy3}

 	$cwin create text $x3 $y -font $fnt10 -text "$_calText(Teilnehmer): " -anchor nw
 	$cwin create text $x4 $y -font $fnt10 -text $uList -width 320 -anchor nw -tag p$lc
	set yl [ lindex [ $cwin bbox p$lc ] 3]
	while {$y< $yl } {incr y $dy3}

 	if { $priv != "" } {
 	    $cwin create text $x4 $y -font $fnt10 -text $priv -anchor nw
 	    incr y $dy3
 	}
	incr y $dy2
	incr lc
	incr count
	
	if { $y > $ymax || $count >= $anz} {
	    if { $y < $ymax } {
		set yl [expr $y -6]
		$cwin create line 0 $yl $cwidth $yl
	    }

	    update
	    set txt [canvas2PS $cwin - 0 0 1]
	    if { $fc ==0 } {
		set p0 [string first "%%Page: " $txt]
		set p1 [string first "%%Trailer" $txt]
		set p2 [string first \n $txt [expr $p0+1]]
		set endTxt [ string range $txt $p1 end]
		set headTxt [string range $txt 0 [expr $p0 -1 ] ]
		incr p1 -1
		set txt "%%Page: $page $page[string range $txt $p2 $p1]"
		set psText "$psText\n\n$txt"
	    } else  {
		set p0 [string first "%%Page: " $txt]
		set p1 [string first "%%Trailer" $txt]
		set p2 [string first \n $txt [expr $p0+1]]
		incr p1 -1
		set txt "%%Page: $page $page[string range $txt $p2 $p1]"
		set psText "$psText\n\n$txt"
	    }
	    
	    incr page
	    incr fc
	    $cwin delete all
	    set ystart [drawListPrintPageHead $cwin [expr $fc+1] $cwidth $cheight]
	    set y $ystart
	}
    }
    
    set p2 [string first "%%Pages" $headTxt]
    set p3 [string first "%%" $headTxt [expr $p2 +4]]
    incr p3 -1
    set headTxt [string replace $headTxt $p2 $p3 "%%Pages: $fc\n"]
    puts $fid $headTxt
    puts $fid $psText
    puts $fid $endTxt
    close $fid
}
proc drawListPrintPageHead {cwin pn w h } {
    global cur_Cal  lbfnt fullUserNames fnt8 progname minkoVersion tyear
    global prStartD prStartM prStartY prStopD prStopM prStopY
    global _printText

    set xm [expr $w /2]
    set xr [expr $w -1]
    set yu [expr $h -10]
    set yc [expr $yu +2]

    $cwin create rectangle 0 0 $xr $yu -outline black -fill white 
    set txt "$_printText(CalList) $_printText(CalListFrom) [date3ToStr $prStartD $prStartM $prStartY 4] $_printText(CalListTo) [date3ToStr $prStopD $prStopM $prStopY 4]"
#    set dates [format "von %2d.%2d.%4d bis %2d.%2d.%4d" $prStartD $prStartM $prStartY $prStopD $prStopM $prStopY ]
    $cwin create text $xm 7 -anchor n -justify center -text $txt -font $lbfnt -tag "calTitle"
    set names ""
    foreach ent $cur_Cal { 
	set names ", $fullUserNames($ent)"
    }
    set txt "$_printText(CalListOf) [string trimleft $names {, }]"
    set txt [strTrunc3 $txt [expr $w -10] $lbfnt [string length $txt]]
    $cwin create text $xm 27 -anchor n -justify center -text $txt -font $lbfnt  -tag "calNames calTitle"
    set y [ lindex [$cwin bbox calNames] 3]
    $cwin create rectangle 0 0 $xr $y  -outline black -fill grey90 -tag "calHead"
    $cwin lower calHead calTitle
    incr y 4
    
    $cwin create text 1 $yc -anchor nw -justify left -text "$_printText(Page) $pn" -font $fnt8
    $cwin create text $xr $yc -anchor ne -justify right -text [format "%c $tyear by $progname $minkoVersion" 169 ] -font $fnt8
    
    return $y
}

