#!/usr/bin/X11/tixwish
#  File: printCal.tcl
# 
#      This file is part of minkowsky
# 
#      Copyright (C) 2001-2002 by Rdiger Goetz
#      Author: Rdiger Goetz <minkowsky@r-goetz.de>
# 
#      Time-stamp: <05-Jun-2002 22:46:04 goetz>
# 
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
# 
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
# 
#      You should have received a copy of the GNU General Public License
#      along with this program; if not, write to the Free Software
#      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#  


proc initPrintProj { } {
    global printProjFormat printProjSplit
    set printProjFormat "all"
    set printProjSplit 0
}

proc printProjDialog { win } {
}

proc printProjOptionsDialog { win } {
    global _printText bfnt bfnt12
    global printProjSplit printProjFormat

    pack $win -side top -fill x -pady 2  -padx 2
    set fwin [$win subwidget frame]

    frame $fwin.p
    pack $fwin.p -side top -fill x -padx 2 -pady 4
    tixOptionMenu $fwin.p.p -options {menubutton.width 40 label.width 12} \
 	-label $_printText(ProjFormat) 
    $fwin.p.p subwidget label configure -font $bfnt
    $fwin.p.p subwidget menubutton configure  -font $bfnt12
    $fwin.p.p add command all        -label $_printText(ProjFmtAll)
    $fwin.p.p add command projOpen   -label $_printText(ProjFmtProjOpen)
    $fwin.p.p add command projClosed -label $_printText(ProjFmtProjClose)
    $fwin.p.p add command freeTask   -label $_printText(ProjFmtFreeTask)
    $fwin.p.p add command allTask    -label $_printText(ProjFmtAllTask)
    $fwin.p.p configure -variable printProjFormat

    pack $fwin.p.p -side left -fill x

    frame $fwin.c
    pack $fwin.c -side top -fill x -padx 2 -pady 4
    checkbutton $fwin.c.c -text "  $_printText(ProjSplit)" -width 60 -anchor nw -justify left\
	-onvalue 0 -offvalue 1 -variable printProjSplit -font $bfnt 
    pack  $fwin.c.c -side left -fill x 
    puts "printProjSplit =$printProjSplit"
}

proc printProjList { file } {
    global  selPrinter cputype printMode  env printFile 
    global  printPageFormat printPageFormats _printText
  


    openMeterWithCancel $_printText(MeterTitle) $_printText(MeterPrepare) 1 .pr
    if { [printProj2Canvas .pr.c.c  $file ] == 1 } {
	send2Printer $file
    }
    closeMeter
    destroy .pr
}

proc printProj2Canvas { cwin file } {
    global bfnt12 fnt10 fnt8 username printPageFormats printBaseFormats printPageFormat
    global projSortMode  projSortDirection tdd_projList tdd_todoList
    global printProjX printProjSplit printProjFormat
    global printPStext printHeadText printEndText printPage
    global meterCanceled _printText bfnt
    global tidPrinted


    set base    [ lindex $printPageFormats($printPageFormat) 1]
    set cwidth  [ lindex $printBaseFormats($base)   0]
    set cheight [ lindex $printBaseFormats($base)   1]
    
    $cwin configure -width  $cwidth
    $cwin configure -height $cheight
    
    set printProjX(7)  [expr $cwidth -   4]
    set printProjX(6)  [expr $cwidth - 100]
    set printProjX(5)  [expr $cwidth - 150]
    set printProjX(4)  [expr $cwidth - 294]
    set printProjX(4t) [expr $cwidth - 288]
    set printProjX(3)  [expr $cwidth - 364]
    set printProjX(2) 32
    set printProjX(1) 8
    set printProjX(0) 4
    set ymax [ expr $cheight -16]

    updateMeter 5 $_printText(MeterPrepare)
    if { $meterCanceled } {
	destroy .pr 
	return 0
    }
    switch -glob $projSortMode {
	id {set pidList [lsort -integer $projSortDirection [array names tdd_projList]]  }
	default {set pidList [lsort -command compareProj $projSortDirection [array names tdd_projList]]  
	}
    }


	
    set printPage 1
    set y [printProjAddFrame $cwin $cwidth $cheight]
    set printPStext ""
    set c  0
    set tag ""

    if { $printProjFormat == "allTask" } {
	set tidList [array names tdd_todoList]
	set count [llength $tidList]
	foreach tid $tidList {
	    set prvTag $tag
	    set tag "todo$tid"
	    updateMeter [expr int(floor(100*double($c)/double($count)))]   $_printText(MeterCreate)
	    set y [printProjAddTodo .pr.c.c $y $cwidth TODO $tid $tag]
	    if {$y > $ymax } {
		# next page and again
		$cwin delete $tag
		set yl [ lindex [ $cwin bbox $prvTag ] 3]
		if { [expr $ymax -$yl ] > 30 } { 
		    $cwin create text [expr int(floor($cwidth/2))] [expr $ymax -20] -text $_printText(ProjMore)\
			-font $bfnt -anchor n -justify center
		}
		printProjCanvas2Str $cwin
		$cwin delete all
		set prvTag ""
		set y [printProjAddFrame $cwin $cwidth $cheight]
		set y [printProjAddTodo .pr.c.c $y $cwidth TODO $tid $tag]
	    }
	    
	    if { $meterCanceled } {
		destroy .pr 
		return 0
	    }
	    incr c
	}
    } else {
	set count [llength $pidList]
	foreach pid $pidList { 
	    set proj $tdd_projList($pid)
	    updateMeter [expr int(floor(100*double($c)/double($count)))]   $_printText(MeterCreate)
	    if { $meterCanceled } {
		destroy .pr 
		return 0
	    }
	    
	    set prvTag $tag
	    if { [lindex $proj 1] == "PROJ" } {
		if { $printProjFormat == "freeTask" } { continue }
		set tag "proj$pid"
		set y [printProjAddProj .pr.c.c $y $cwidth  $cheight $pid $tag $printProjSplit $ymax]
	    } elseif { [lindex $proj 1] == "TODO" } {
		if { $printProjFormat == "projOpen" || $printProjFormat == "projClosed" } { continue }
		set list [lindex $proj 12]
		set ent [lindex $list 0]
		set tid [lindex $ent 0]
		set tag "todo$tid"
		set y [printProjAddTodo .pr.c.c $y $cwidth TODO $tid $tag]
	    }
	    
	    if { $meterCanceled } {
		destroy .pr 
		return 0
	    }
	    
	    if {$y > $ymax } {
		# next page and again
		$cwin delete $tag
		set yl [ lindex [ $cwin bbox $prvTag ] 3]
		if { [expr $ymax -$yl ] > 30 } { 
		    $cwin create text [expr int(floor($cwidth/2))] [expr $ymax -20] -text $_printText(ProjMore)\
			-font $bfnt -anchor n -justify center
		}
		printProjCanvas2Str $cwin
		$cwin delete all
		set prvTag ""
		set y [printProjAddFrame $cwin $cwidth $cheight]
		if { [lindex $proj 1] == "PROJ" } {
		    set tag "proj$pid"
		    set y [printProjAddProj .pr.c.c $y $cwidth $cheight  $pid $tag 1 $ymax]
		} elseif { [lindex $proj 1] == "PROJ" } {
		    set list [lindex $proj 12]
		    set ent [lindex $list 0]
		    set tid [lindex $ent 0]
		    set tag "todo$tid"
		    set y [printProjAddTodo .pr.c.c $y $cwidth TODO $tid $tag]
		}
	    }
	    
	    if { $meterCanceled } {
		destroy .pr 
		return 0
	    }
	    incr c
	}
    }
    if { $y > 28 } {
	printProjCanvas2Str $cwin
    }

    updateMeter 90 $_printText(MeterPrepare)
    if { $meterCanceled } {
	destroy .pr 
	return 0
    }
    set fid [open $file "w"]
    set p2 [string first "%%Pages" $printHeadText]
    set p3 [string first "%%" $printHeadText [expr $p2 +4]]
    incr p3 -1
    set headTxt [string replace $printHeadText $p2 $p3 "%%Pages: [expr $printPage-1]\n"]


    puts $fid $headTxt
    puts $fid $printPStext
    puts $fid $printEndText
    close $fid

    updateMeter 95 $_printText(Meter2Printer)
    if { $meterCanceled } {
	destroy .pr 
	return 0
    }
    return 1
}

proc printProjCanvas2Str {cwin  } {
    global printPStext printHeadText printEndText printPage

    set txt [canvas2PS $cwin - 0 0 1]
    set p0 [string first "%%Page: " $txt]
    set p1 [string first "%%Trailer" $txt]
    set p2 [string first \n $txt [expr $p0+1]]
    
    if {$printPage == 1 } {
	set printEndText [ string range $txt $p1 end]
	set printHeadText [string range $txt 0 [expr $p0 -1 ] ]
    }
    incr p1 -1
    set txt "%%Page: $printPage $printPage [string range $txt $p2 $p1]"

    if {$printPage == 1 } {
	set printPStext "$txt"
    } else {
	set printPStext "$printPStext\n\n$txt"
    }
    incr printPage
}
    
proc printProjAddFrame {cwin cwidth cheight} {
    global bfnt18 fnt8 tyear progname minkoVersion username fullUserNames printPage _printText cur_Cal

    set yh [expr $cheight -16]
    $cwin create rectangle 0 0 $cwidth $yh -fill white -outline black
    $cwin create rectangle 0 0 $cwidth 24 -fill gray80 -outline black

    set fwho [calUserList2NameList $cur_Cal]
    set txt "$_printText(ProjTitle) $_printText(ProjTitleOf) $fwho"
    set txt [strTrunc3 $txt [expr $cwidth -8] $bfnt18 [string length  $txt]]
    $cwin create text [expr int(floor($cwidth/2))] 4 -text $txt	-font $bfnt18 -anchor n -justify center

    $cwin create text 0 [expr $yh +2] -text "$_printText(Page) $printPage" -font $fnt8 -anchor nw
    $cwin create text $cwidth [expr $yh +2] -text [format "%c $tyear by $progname $minkoVersion" 169 ]\
	-font $fnt8 -anchor ne -justify right
    return 28
    
}

proc printProjAddProj {cwin y cwidth cheight pid tag split ymax } {
    global tdd_projList projSortMode  projSortDirection printProjX _projText _printText bfnt
    global printProjFormat

    set proj $tdd_projList($pid)
    set ustr ""
    set init [lindex $proj 9]
    foreach usr [lindex $proj 8] {
	if { $usr == $init} {
	    set ustr [format "%s, %s(%s)" $ustr [lindex $usr 0] $_projText(mainInit)]
	} else {
	    set ustr [format "%s, %s" $ustr [lindex $usr 0]]
	}
    }
    set ustr [string trim $ustr ", "]
    set y [ printProjAddItem .pr.c.c $y $cwidth PROJ [lindex $proj 4] [lindex $proj 5] \
		"" [date2lstr [lindex $proj 7] ] [lindex $proj 11] \
		$ustr [lindex $proj 10] [lindex $proj 14] $tag ]

    set tidList ""
    foreach ent [lindex $proj 12] {
	set tid [ lindex $ent 0]
	lappend tidList $tid
    }
    
    set tList $tidList
    switch -glob $projSortMode {
	id {set tList [lsort -integer $projSortDirection $tidList]  }
	default {set tList [lsort $projSortDirection -command compareTodo $tidList]  
	}
    }
    incr y 2

    if { $printProjFormat != "projClosed" } {
	set tag3 $tag
	foreach tid $tList {
	    set prvTag $tag3
	    set tag3 "ptodo$tid"
	    set tag2 "$tag $tag3"
	    set y [ printProjAddTodo $cwin $y $cwidth PROJTODO $tid $tag2]
	    if { $split == 1 } {
		if { $y > $ymax } {
		    $cwin delete $tag3
		    set yl [ lindex [ $cwin bbox $prvTag ] 3]
		    if { [expr $ymax -$yl ] > 30 } { 
			$cwin create text [expr int(floor($cwidth/2))] [expr $ymax -20]\
			    -text $_printText(ProjMore) -font $bfnt -anchor n -justify center
		    }
		    printProjCanvas2Str $cwin
		    $cwin delete all
		    set y [printProjAddFrame $cwin $cwidth $cheight]
		    set y [ printProjAddTodo $cwin $y $cwidth PROJTODO $tid $tag2]
		}
	    }
	    incr y 2
	}
    }
    $cwin create line $printProjX(0) [expr $y -2] $printProjX(7) [expr $y -2]  -tag $tag

    incr y 4
    return $y
}

proc printProjAddTodo { cwin y cwidth type tid tag } {
    global tdd_todoList tdd_todoCompl tdd_todoDring _projText

    set todo $tdd_todoList($tid)
    set ustr ""
    set init [lindex $todo 8]
    foreach usr [lindex $todo 7] {
	if { $usr == $init} {
	    set ustr [format "%s, %s(%s)" $ustr [lindex $usr 0] $_projText(mainInit)]
	} else {
	    set ustr [format "%s, %s" $ustr [lindex $usr 0]]
	}
    }
    set ustr [string trim $ustr ", "]
    return [ printProjAddItem $cwin $y $cwidth $type [lindex $todo 2] [lindex $todo 3] \
		 [date2lstr [lindex $todo 4]] [date2lstr [lindex $todo 5]] \
		 $tdd_todoCompl($tid) $ustr  $tdd_todoDring($tid) [lindex $todo 11] $tag ]
}

proc printProjAddItem { cwin y cwidth  type stat  title start stop complete users  dring descr tag } {
    global printProjX dringlichStr TODOstats
    global bfnt16 bfnt bfnt12 fnt12 fnt10 ifnt12
    global _projText
    
    set dx  [ expr $printProjX(5) -$printProjX(1) -4]
    set dx2 [ expr $printProjX(5) -$printProjX(2) -4]
    set dx3 [ expr $printProjX(5) -$printProjX(3) -4]
    set dx4 [ expr $printProjX(7) -$printProjX(5) -4]
    set dx5 [ expr $printProjX(7) -$printProjX(4t)-4]
    if {$type == "PROJ" } {
	set txt [strTrunc3 $title $dx  $bfnt [string length $title]]
	$cwin create text $printProjX(1) $y -text $txt -font $bfnt16 -anchor nw -tag $tag
	incr y 6
	set tag1 $tag
    } elseif {$type == "PROJTODO" } {
 	set txt [strTrunc3 $title $dx2 $bfnt12 [string length $title]]
	$cwin create text $printProjX(2) $y -text $txt  -font $bfnt12 -anchor nw -tag $tag
	incr y 2
	set tag1 [lindex $tag 0]
    } else {
 	set txt [strTrunc3 $title $dx $bfnt12 [string length $title]]
	$cwin create text $printProjX(1) $y -text $txt  -font $bfnt12 -anchor nw -tag $tag
	incr y 2
	set tag1 $tag
    }
    $cwin create text $printProjX(5) $y -text $_projText(mainStatus)   -font $ifnt12 -anchor nw -tag $tag
    $cwin create text $printProjX(6) $y -text ": $TODOstats($stat)"    -font $fnt12  -anchor nw -tag $tag
    set y [ lindex [ $cwin bbox $tag1 ] 3]

    if { $type != "PROJ" } {
	$cwin create text $printProjX(3) $y -text $_projText(mainStart)    -font $ifnt12 -anchor nw -tag $tag
	$cwin create text $printProjX(4) $y -text ": $start"            -font $fnt12  -anchor nw -tag $tag
    }
    $cwin create text $printProjX(5) $y -text $_projText(todoPrintDring) -font $ifnt12 -anchor nw -tag $tag
    $cwin create text $printProjX(6) $y -text ": [lindex $dringlichStr $dring]" -font $fnt12  -anchor nw -tag $tag
    set y [ lindex [ $cwin bbox $tag1 ] 3]
    set y [ lindex [ $cwin bbox $tag1 ] 3]

    $cwin create text $printProjX(3) $y -text $_projText(mainEnd)      -font $ifnt12 -anchor nw -tag $tag
    $cwin create text $printProjX(4) $y -text ": $stop"                -font $fnt12  -anchor nw -tag $tag
    $cwin create text $printProjX(5) $y -text $_projText(mainProgress) -font $ifnt12 -anchor nw -tag $tag
    $cwin create text $printProjX(6) $y -text ": $complete %"          -font $fnt12  -anchor nw -tag $tag
    set y [ lindex [ $cwin bbox $tag1 ] 3]


    $cwin create text $printProjX(3) $y -text $_projText(mainWorker)   -font $ifnt12 -anchor nw -tag $tag
    $cwin create text $printProjX(4) $y -text ": "                     -font $fnt12  -anchor nw -tag $tag
    $cwin create text $printProjX(4t) $y -text $users  -width $dx5     -font $fnt12  -anchor nw -tag $tag
    set y [ lindex [ $cwin bbox $tag1 ] 3]

    $cwin create text $printProjX(3) $y -text $_projText(projDescr)     -font $ifnt12 -anchor nw -tag $tag
    $cwin create text $printProjX(4) $y -text ":"                       -font $fnt12  -anchor nw -tag $tag
    $cwin create text $printProjX(4t) $y -text $descr -width $dx5       -font $fnt12  -anchor nw -tag $tag
    set y [ lindex [ $cwin bbox $tag1 ] 3]

    if {$type == "PROJTODO" || $type == "PROJ" } {
	$cwin create line $printProjX(3) $y $printProjX(7) $y -tag $tag
    } elseif {$type == "TODO" } {
	$cwin create line $printProjX(0) $y $printProjX(7) $y -tag $tag
    }
    incr y 2
    return $y
}